#include <qsocket.h>
#include <qobject.h>
#include <ksystemtray.h>
#include <kpopupmenu.h>
#include <qstring.h>
#include <qpixmap.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>
#include <errno.h>
#include <kapplication.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <kpassivepopup.h>
#include <kmessagebox.h>
#include <qprocess.h>
#include <qfile.h>
#include <klocale.h>

#include "client.h"
#include "gprswindow.h"
#include "adslwindow.h"
#include "connect_socket.h"


#define	SERVSOCK	"/var/run/adslserv.sock"
#define	NAME		"ADSL Manager"
#define ICON		"/usr/share/icons/hicolor/16x16/apps/kaddressbook.png"
#define TIMER		3000

Client::Client( QString locale )
{

	s = socket( PF_LOCAL, SOCK_STREAM, 0 );

	sock = new QSocket( );

	memset( &servsock, 0, sizeof( servsock ) );
	
	servsock.sun_family = AF_LOCAL;
	strcpy( servsock.sun_path, SERVSOCK );

	connect_socket( s, ( struct sockaddr * )&servsock, SUN_LEN( &servsock ) );	

	sock->setSocket( s );
	
	modem_status = 2;
	connection = 0;
	new_modem = 1;
	first_config = 0;
	connecting = 0;
	share_type = 0;
	
	tray = new KSystemTray( );
	tray->setPixmap( QPixmap( ICON ) );
	tray->contextMenu( )->changeTitle( -2, NAME );

	tray->hide( );

	timer = new QTimer;
	process = new QProcess;

	gprsWindow = new GPRSWindow( QPixmap( ICON ) );
	adslWindow = new ADSLWindow( locale, QPixmap( ICON ) );
	
	QObject::connect( tray->contextMenu( ), SIGNAL( activated( int ) ), this, SLOT( choice( int ) ) );	
	QObject::connect( tray, SIGNAL( quitSelected( ) ), this, SLOT( quit( ) ) );
	QObject::connect( sock, SIGNAL( readyRead( ) ), this, SLOT( readSockData( ) ) );
	QObject::connect( process, SIGNAL( readyReadStdout( ) ), this, SLOT( readStatus( ) ) );
	QObject::connect( gprsWindow, SIGNAL( data( int, QString ) ), this, SLOT( gprsData( int, QString ) ) );
	QObject::connect( adslWindow, SIGNAL( data( QString, QString, QString, int, int, QString ) ), this, SLOT( adslData( QString, QString, QString, int, int, QString ) ) );
	QObject::connect( adslWindow, SIGNAL( closeCancel( ) ), this, SLOT( confWindowCancel( ) ) );
	QObject::connect( gprsWindow, SIGNAL( closeCancel( ) ), this, SLOT( confWindowCancel( ) ) );

	process->addArgument( "/usr/share/adsl-manager-client/bin/status.sh" );

	setUpTray( );

	command( "MODEM" );

}

Client::~Client( )
{
	// destructor here
}

void Client::trayConnect( )
{
	tray->contextMenu( )->setItemVisible( 1, TRUE );
	tray->contextMenu( )->setItemVisible( 2, FALSE );
	tray->contextMenu( )->setItemVisible( 3, TRUE );
	tray->contextMenu( )->setItemVisible( 4, TRUE );
	
	if( share_type != 0 )
	{
		tray->contextMenu( )->setItemVisible( 5, FALSE );
		tray->contextMenu( )->setItemVisible( 6, TRUE );
	}
	else
	{
		tray->contextMenu( )->setItemVisible( 5, TRUE );
		tray->contextMenu( )->setItemVisible( 6, FALSE );
	}
}

void Client::trayDisconnect( )
{
	tray->contextMenu( )->setItemVisible( 1, FALSE );
	tray->contextMenu( )->setItemVisible( 2, TRUE );
	tray->contextMenu( )->setItemVisible( 3, TRUE );
	tray->contextMenu( )->setItemVisible( 4, TRUE );
	
	if( share_type != 0 )
	{
		tray->contextMenu( )->setItemVisible( 5, FALSE );
		tray->contextMenu( )->setItemVisible( 6, TRUE );
	}
	else
	{
		tray->contextMenu( )->setItemVisible( 5, TRUE );
		tray->contextMenu( )->setItemVisible( 6, FALSE );
	}
}

void Client::readSockData( )
{
	QByteArray data;
	data = sock->readAll( );

	if( strncmp( data.data( ), "WINDOW", 6 ) == 0 )
	{
		char * t;
		t = index( data.data( ), ' ' );
		t++;
		if( strncmp( t, "1", 1 ) == 0 )
		{
			first_config = 1;
			gprsWindow->showWindow( );
		}
		if( strncmp( t, "2", 1 ) == 0 )
		{
			first_config = 1;
			adslWindow->showWindow( 2 );
		}
		if( strncmp( t, "3", 1 ) == 0 )
		{
			first_config = 1;
			adslWindow->showWindow( 3 );
		}
		return;
	}

	if( strncmp( data.data( ), "MODEM", 5 ) == 0 )
	{
		char * t;
		t = index( data.data( ), ' ' );
		t++;
		
		if( strncmp( t, "0", 1 ) == 0 )
		{
			tray->hide( );
		}
	
		if( strncmp( t, "1", 1 ) == 0 )
		{	
			trayConnect( );
			tray->show( );
			KPassivePopup::message( NAME, tr( "New modem found" ), tray );
		}
		return;
	}
	
	if( strcmp( last_command, "STATUS" ) == 0 )
	{

		if( strncmp( data.data( ), "0", 1 ) == 0  )
			trayConnect( );
	
		if( strncmp( data.data( ), "1", 1 ) == 0  )
			trayDisconnect( );
		
		
	}

	if( strcmp( last_command, "WINDOW" ) == 0 )
	{
		if( strncmp( data.data( ), "1", 1 ) == 0 )
		{
			gprsWindow->showWindow( );
		}
		else if( strncmp( data.data( ), "2", 1 ) == 0 )
		{
			adslWindow->showWindow( 2 );
		}
		else if( strncmp( data.data( ), "3", 1 ) == 0 )
		{
			adslWindow->showWindow( 3 );
		}
		else command( "STATUS" );
	}

	if( strcmp( last_command, "MODEM" ) == 0 )
	{
		if( strncmp( data.data( ), "0", 1 ) == 0 ) 
		{
			modem_status = 0;
			tray->hide( );

		}

		if( strncmp( data.data( ), "1", 1 ) == 0 ) 
		{
			trayConnect( );
			tray->show( );
			KPassivePopup::message( NAME, tr( "New modem found" ), tray );
			modem_status = 1;
		}
	}

	if( strcmp( last_command, "CONNECTION" ) == 0 )
	{
		if( strncmp( data.data( ), "1", 1 ) == 0 )
		{
			trayDisconnect( );
			tray->show( );
			KPassivePopup::message( NAME, tr( "Connected" ), tray );
			connection = 1;
			connecting = 0;
			disconnect( timer, SIGNAL( timeout( ) ), this, SLOT( checkConnection( ) ) );
		}
		
		if( strncmp( data.data( ), "2", 1 ) == 0 )
		{
			KPassivePopup::message( NAME, tr( "Not connected" ), tray );
			connecting = 0;
			disconnect( timer, SIGNAL( timeout( ) ), this, SLOT( checkConnection( ) ) );
		}
	}

	if( strcmp( last_command, "DISCONNECT" ) == 0 )
	{
		if( strncmp( data.data( ), "1", 1 ) == 0 )
		{
			trayConnect( );
			KPassivePopup::message( NAME, tr( "Disconnected" ), tray );
			
			connection = 0;
		}

		if( strncmp( data.data( ), "0", 1 ) == 0 )
			KPassivePopup::message( NAME, tr( "Could not disconnect" ), tray );

		disconnect( timer, SIGNAL( timeout( ) ), this, SLOT( checkConnection( ) ) );
	}

	if( strcmp( last_command, "MODEM_TYPE" ) == 0 )
	{
		if( strncmp( data.data( ), "1", 1 ) == 0 )
		{
			gprsWindow->showWindow( );	
		}

		if( strncmp( data.data( ), "2", 1 ) == 0 )
		{
			adslWindow->showWindow( 2 );
		}
		if( strncmp( data.data( ), "3", 1 ) == 0 )
		{
			adslWindow->showWindow( 3 );
		}
	}
		
	if( strcmp( last_command, "SHARE_DHCP" ) == 0 || strcmp( last_command, "SHARE_NODHCP" ) == 0 )
	{
		if( strncmp( data.data( ), "1", 1 ) == 0 )
		{
			tray->contextMenu( )->setItemVisible( 5, FALSE );
			tray->contextMenu( )->setItemVisible( 6, TRUE );
			KPassivePopup::message( NAME, tr( "Started sharing connection" ), tray );
		}
		else
			KPassivePopup::message( NAME, tr( "Could not start sharing connection" ), tray );
	}
	
	if( strcmp( last_command, "SHARE_QUIT_DHCP" ) == 0 || strcmp( last_command, "SHARE_QUIT_NODHCP" ) == 0 )
	{
		if( strncmp( data.data( ), "1", 1 ) == 0 )
		{
			tray->contextMenu( )->setItemVisible( 5, TRUE );
			tray->contextMenu( )->setItemVisible( 6, FALSE );
			KPassivePopup::message( NAME, tr( "Stopped sharing connection" ), tray );
			share_type = 0;
		}
		else
			KPassivePopup::message( NAME, tr( "Could not stop sharing connection" ), tray );

	}

}

void Client::gprsData( int pin, QString isp )
{
	char buff[20];
	snprintf( buff, 20, "%s %d %s", "GPRS", pin, isp.ascii( ) );

	command( buff );
	sleep( 1 );
	command( "DISCONNECT" );
	sleep( 1 );
	command( "CONNECT" );

	first_config = 0;
}

void Client::adslData( QString type, QString user, QString pass, int vci, int vpi, QString path )
{
	char buff[1024];
	snprintf( buff, 1024, "%s %s %s %s %d %d %s", "ADSL", type.ascii( ), user.ascii( ), pass.ascii( ), vci, vpi, path.ascii( ) );

	command( buff );
	sleep( 1 );
	command( "DISCONNECT" );
	sleep( 1 );
	command( "CONNECT" );
	first_config = 0;
	trayConnect( );
	tray->show( );
	KPassivePopup::message( NAME, tr( "Connecting..." ), tray );
}

// COMMANDS:
// CONNECT - connects the ADSL modem
// DISCONNECT - disconnects the ADSL modem
// STATUS - gets status of the connection: 1 - connected; 0 - disconnected
// DISCONNECT_QUIT - disconnects the ADSL modem without waiting for reply

void Client::command( char * comm )
{


		if( strcmp( comm, "CONNECT" ) == 0 && connecting )
			return;


		last_command = comm;
		sock->writeBlock( comm, strlen( comm ) );
		sock->flush( );

		if( strcmp( comm, "CONNECT" ) == 0 && !connecting )
		{
			disconnect( timer, SIGNAL( timeout( ) ), this, SLOT( checkModem( ) ) );
			connect( timer, SIGNAL( timeout( ) ), this, SLOT( checkConnection( ) ) );
			timer->start( TIMER );
			connecting = 1;
		}


}

void Client::choice( int id ) 
{

	switch( id )
	{
		int c;
		case 1:
			command( "CONNECT" );
			KPassivePopup::message( NAME, tr( "Connecting..." ), tray );
			break;
		case 2:
			command( "DISCONNECT" );
			KPassivePopup::message( NAME, tr( "Disconnecting..." ), tray );
			break;
			

		case 3:
			process->start( );
			break;

		case 4:
			command( "MODEM_TYPE" );			
			break;

		case 5:
			c = KMessageBox::questionYesNo( 0, tr( "Do you want to enable automatic network configuration (DHCP) ?" ), NAME );
			switch( c )
			{
				case 3:
					command( "SHARE_DHCP" );
					share_type = 3;
					break;
				case 4:
					command( "SHARE_NODHCP" );
					share_type = 4;
					break;
			}
			break;

		case 6:
			switch( share_type )
			{
				case 3:
					command( "SHARE_QUIT_DHCP" );
					break;
				case 4:
					command( "SHARE_QUIT_NODHCP" );
					break;
			}	
			break;
	}

}

void Client::setUpTray( )
{

	if( ! tray->contextMenu( )->isItemActive( 1 ) && ! tray->contextMenu( )->isItemActive( 2 ) )
	{	
		tray->contextMenu( )->insertItem( QIconSet( QPixmap( "/usr/share/adsl-manager-client/pixmaps/tick.gif" ) ), tr( "Connect" ), 1 );
		tray->contextMenu( )->insertItem( QIconSet( QPixmap( "/usr/share/adsl-manager-client/pixmaps/cross.gif" ) ), tr( "Disconnect" ), 2 );
		tray->contextMenu( )->insertItem( tr( "Status" ), 3 );
		tray->contextMenu( )->insertItem( tr( "Configuration" ), 4 );
		tray->contextMenu( )->insertItem( tr( "Share connection" ), 5 );
		tray->contextMenu( )->insertItem( tr( "Quit sharing connection" ), 6 );

		tray->contextMenu( )->setItemVisible( 6, FALSE );
//		tray->show( );
	}

}

void Client::quit( )
{
	if( connection == 1 )
		command( "DISCONNECT_QUIT" );	
	sock->close( );
	kapp->quit( );
	
}

void Client::checkModem( )
{
	command( "MODEM" );
}

void Client::checkConnection( )
{
	command( "CONNECTION" );
}

void Client::readStatus( )
{
	QByteArray data = process->readStdout( );

	KMessageBox::information( 0, QString( data ), NAME );
}

void Client::confWindowCancel( )
{
	if( first_config )
	{
		tray->contextMenu( )->setItemVisible( 1, FALSE );
		tray->contextMenu( )->setItemVisible( 2, FALSE );
		tray->contextMenu( )->setItemVisible( 3, FALSE );
		tray->contextMenu( )->setItemVisible( 5, FALSE );
		tray->contextMenu( )->setItemVisible( 6, FALSE );
	}

	tray->show( );
}
